/* depends on:
	jquery core 1.4.2 or later <http://jquery.com>
	jquery query object plugin <http://plugins.jquery.com/project/query-object>
*/


var harvest_results;
var harvest_filters_list;


function Filter (dom_node) {
	/* Attaches to a .filter object in the document. Handles events
	   specific to each type of filter and provides some extra data
	   and utilities. */
	
	var filter = this; /* so we can access this from event handlers */
	
	$(dom_node).data('filter',this); /* so we can get here from the JQuery object */
	
	this.full_name = $(dom_node).attr('data-filter-fullname');
	this.parent_filter_node = $(dom_node).parent().closest('.filter');
	this.container = $(dom_node).closest('li'); /* the list item in the parent that this belongs to */
	this.value_node = $(dom_node).children('.filter-value');
	
	
	this.is_class = function (class_name) {
		return $(dom_node).hasClass(class_name);
	}
	
	
	this.get_value_serialized = function () {
		/* Returns the value of this filter, serialized in the same
		   format we would use in Harvest's Django end.
		   Does nothing by default. Implement this in each
		   "class"-specific initializer below. */
		
		return null;
	}
	
	this.value_changed = function () {
		/* Should be called when the filter's value has changed.
		   This posts the new value to the global Results object */
		
		value = this.get_value_serialized();
		if ( value != null ) {
			harvest_results.post_query(this.full_name, value);
		}
	}
	
	
	/********/
	/* Ugly hacks for talking to the parent node */
	
	this.get_parent_filtergroup = function () {
		/* Returns the FilterGroup that is a parent of this filter.
		   Returns null if the parent doesn't exist or isn't a FilterGroup. */
		parent = null;
		if ( this.parent_filter_node ) {
			parent_test = this.parent_filter_node.data('filter');
			if ( parent_test && parent_test.is_class('filtergroup') ) parent = parent_test;
		} 
		return parent;
	}
	
	
	this.force_enable_prelight_add = function () {
		/* Tells the parent (if there is one) to create a prelight for
		   enabling this item.
		   We assume the parent is a FilterGroup, which is probably right. */
		parent_filter = this.get_parent_filtergroup();
		if ( parent_filter && ! parent_filter.item_is_selected(this.container) ) {
			/* only do this if we weren't already selected */
			parent_filter.toggle_item_prelight_add(this.container);
		}
	}
	
	this.force_enable_prelight_clear = function () {
		/* Tells the parent (if there is one) to remove the prelight for
		   enabling this item.
		   We assume the parent is a FilterGroup. */
		parent_filter = this.get_parent_filtergroup()
		
		if ( parent_filter ) parent_filter.toggle_item_prelight_clear(this.container);
	}
	
	
	this.force_enable = function () {
		/* Tells the parent (if there is one) to enable this item,
		   because it said so!
		   We assume the parent is a FilterGroup. */
		parent_filter = this.get_parent_filtergroup();
		if ( parent_filter && ! parent_filter.item_is_selected(this.container) ) {
			parent_filter.enable_item(this.container);
		}
	}
	
	
	/********/
	/* "Class"-specific initialization stuff. Setting up functions and
	   handlers in filter.value_node */
	
	/* TODO: stick this in its own namespace */
	if ( this.is_class('choicefilter') || this.is_class('filtergroup') ) {
		var list_items = filter.value_node.children('ul').children('li');
		
		
		filter.item_is_selected = function (item) {
			return ( item.attr('data-selected') != null );
		}
		
		this.toggle_item_prelight_add = function (item) {
			if ( filter.item_is_selected(item) ) {
				item.addClass('prelight-off');
			} else {
				item.addClass('prelight-on');
				item.children('.checkbox').html('&#10003;'); /* decoration :) */
				filter.force_enable_prelight_add();
			}
		}
		
		this.toggle_item_prelight_clear = function (item) {
			/* remove the decoration in the checkbox if we added it */
			if ( ! filter.item_is_selected(item) ) {
				item.children('.checkbox').html('');
			}
			
			item.removeClass('prelight-on');
			item.removeClass('prelight-off');
			filter.force_enable_prelight_clear();
		}
		
		
		this.enable_item = function (item) {
			item.attr('data-selected','');
			item.children('.checkbox').html('&#10003;');
			filter.force_enable();
			
			filter.value_changed();
		}
		
		this.disable_item = function (item) {
			item.removeAttr('data-selected');
			item.children('.checkbox').html('');
			
			filter.value_changed();
		}
		
		this.toggle_item = function (item) {
			if ( filter.item_is_selected(item) ) {
				filter.disable_item(item);
			} else {
				filter.enable_item(item);
			}
		}
		
		
		this.get_item_toggle_node = function (item) {
			/* returns the node(s) used to toggle the given item in this filter */
			/* TODO: make the DOM layout nice so we don't need two cases here */
			var toggle_node;
			if ( filter.is_class('filtergroup') ) {
				/* using child selector doesn't seem to work on item.filter()... */
				toggle_node = item.children('.filter').children('.filter-label').children('a.item-toggle');
			} else {
				toggle_node = item.children('a.item-toggle');
			}
			/*checkbox = item.children('.checkbox');
			toggle_node = toggle_node.add(checkbox);*/
			return toggle_node;
		}
		
		
		/* setup the event handlers we described on all the items */
		list_items.each(function () {
			var item = $(this);
			toggle_node = filter.get_item_toggle_node(item);
			toggle_node.bind('mouseenter focusin', function () { filter.toggle_item_prelight_add(item); } );
			toggle_node.bind('mouseleave focusout', function () { filter.toggle_item_prelight_clear(item); } );
			
			toggle_node.bind('click', function () {
				filter.toggle_item(item);
				filter.toggle_item_prelight_clear(item);
				
				return false; /* stops link from being followed */
			} );
		});
		
		
		this.get_value_serialized = function () {
			var item_ids = Array();
			/* we add the data-item-id attribute of all selected items */
			list_items.each(function () {
				if ($(this).attr('data-selected') != null) {
					item_ids.push($(this).attr('data-item-id'));
				}
			} );
			return item_ids.join(',');
		}
	}
}


function Results (dom_node) {
	/* Attached to the single #results pane in the document. Receives
	   new query parameters from Filter objects, grabs and displays new
	   results. */
	
	/* The current_query variable is set with the current querystring,
	   but some filters could already have been set with Harvest's
	   default parameters while the querystring doesn't mention them.
	   On its own, this isn't a problem, but it may lead to some odd
	   behaviour. We should fix it by setting current_query to include
	   all of the defaults in Harvest's Django portion. */
	
	var results = this;
	
	this.query_countdown = 0;
	
	this.current_query = $.query;
	this.future_query = null;
	
	this.results_pane = $(dom_node);
	this.results_url = $(dom_node).attr('data-results-url');
	
	this.running_xhr = null;
	
	
	this.load_results = function (query) {
		/* Begins loading results for the given query */
		querystring = unescape(query.toString()).substring(1); /* FIXME: this is horrible. Let's use a dict at this point instead */
		if ( this.running_xhr != null ) {
			this.running_xhr.abort();
		}
		this.running_xhr = $.get(this.results_url, querystring,
			function (data) {
				results.results_pane.html(data);
				/* TODO: detect errors here and handle appropriately */
				this.running_xhr = null;
			});
	}
	
	
	this.timer_finish = function () {
		/* When the timer ends we start our new HttpRequest with
		   future_query, move future_query to current_query, and do
		   other bits of cleanup. */
		
		this.current_query = this.future_query;
		this.future_query = null;
		
		if ( this.current_query ) {
			/*console.log(unescape(this.current_query.toString()));*/
			this.load_results(this.current_query);
		}
	}
	
	this.reset_timer = function () {
		/* starts, or restarts, the query countdown timer.
		   The next query will be run in 1.5 seconds. */
		
		this.stop_timer();
		this.query_countdown = setTimeout(
			function (results) {
				results.timer_finish();
			}, 1500, this);
	}
	
	this.stop_timer = function () {
		/* Stops the query countdown timer. */
		
		clearTimeout(this.query_countdown);
		this.query_countdown = 0;
	}
	
	this.post_query = function (key, value) {
		/* build the new query from the existing one if it exists */
		var query;
		this.future_query ? query = this.future_query : query = this.current_query;
		
		query = query.set(key, value);
		
		/* no compare function for the query string object so this is
		   the easiest approach, though a tad inefficient */
		if ( query.toString() == this.current_query.toString() ) {
			/* the user has reverted future_query. So, stop the timer
			   and set future_query back to null, awaiting new input */
			this.stop_timer();
			this.future_query = null;
		} else {
			/* start the timer from 0, waiting to apply the new query */
			this.reset_timer();
			this.future_query = query;
		}
	}
}



function create_filters () {
	/* Creates all filters for the document at once. */
	
	var filters_list = new Array();
	
	$('#filters .filter').each(function (i) {
		filters_list += new Filter(this);
	});
	
	return filters_list;
}

function create_results () {
	return new Results($('#results'));
}



$(document).ready(function () {
	harvest_results = create_results();
	harvest_filters_list = create_filters();
});
