from harvest.filters import filters, containers
import models

class PkgNameFilter(filters.EditFilter):
    def process_queryset(self, queryset):
        return queryset.filter(name__startswith = self.get_value())

class PkgSetFilter(filters.ChoiceFilter):
    def __init__(self, id_str):
        choices_dict = dict()
        for s in models.PackageSet.objects.all():
            choices_dict[s.name] = s
        super(PkgSetFilter, self).__init__(id_str, choices_dict)
    
    def process_queryset(self, queryset):
        return queryset.filter(packagesets__in=self.get_selected_items())



class OppFeaturedFilter(filters.Filter):
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__featured=True)

class OppListFilter(filters.ChoiceFilter):
    def __init__(self, id_str):
        choices_dict = dict()
        for l in models.OpportunityList.objects.all():
            choices_dict[l.name] = l
        super(OppListFilter, self).__init__(id_str, choices_dict)
    
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__in=self.get_selected_items())


#we don't really need to create a special type here, but it may be handy
class HarvestFilters(containers.FilterSystem):
    def __init__(self):
        super(HarvestFilters, self).__init__(
            [
                filters.FilterGroup("pkg", [
                    PkgNameFilter("name"),
                    PkgSetFilter("set")
                ] ),
                filters.FilterGroup("opp", [
                    OppFeaturedFilter("featured"),
                    OppListFilter("list")
                ] )
            ],
            default_parameters = { "pkg" : "name,set",
                                   "pkg:name" : "ged",
                                   "pkg:set" : "ubuntu-desktop" }
        )

